package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.generated.messages.Messages;
import ca.canadapost.cpcdp.ncshipping.generated.ncshipment.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetNonContractShipments Canada Post service.
 * <p>
 * The GetNonContractShipments service invokes a search for previously created 
 * shipments. It is used to recover from communication failures or hardcopy print 
 * failures. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetNCShipments Request.  <br>
 *  USAGE:<pre>
 *        GetNCShipmentsClient client = new GetNCShipmentsClient(username, password);
 *        ClientResponse response = client.getNCShipments();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetNCShipmentsClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Returns_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
		
    	// REST URI
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/ncshipment?from=201112010000&to=201203010000";		
		
		// Execute GetNCShipments Request
        GetNCShipmentsClient myClient = new GetNCShipmentsClient(username, password);
        ClientResponse resp = myClient.getNCShipments();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(NonContractShipments.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches NCShipments schema.
            if (entity instanceof NonContractShipments) {
            	NonContractShipments ncshipments = (NonContractShipments) entity;
            	if (ncshipments.getLinks().isEmpty() == false ) {
	                for (Iterator<Link> iter = ncshipments.getLinks().iterator(); iter.hasNext();) {
	                	Link link = (Link) iter.next();
	                    System.out.println(link.getRel() + ": " + link.getHref());
	                }            	
            	} else {
            		System.out.println("No shipments returned.");
            	}
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetNCShipmentsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getNCShipments() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.ncshipment-v4+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
