package ca.canadapost.cpcdp.ncshipping;


import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import javax.xml.bind.JAXBContext;

import ca.canadapost.cpcdp.ncshipping.generated.ncshipment.NonContractShipmentRefundRequest;
import ca.canadapost.cpcdp.ncshipping.generated.ncshipment.NonContractShipmentRefundRequestInfo;
import ca.canadapost.cpcdp.ncshipping.generated.messages.Messages;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.client.config.DefaultClientConfig;

/**
 * Sample code for the NCShipmentRefundRequest Canada Post service.
 * <p>
 * The NCShipmentRefundRequest service is used to request a refund for 
 * a shipment that has been transmitted. Making this call indicates 
 * that the previously printed label is spoiled or will otherwise not be used.
 * 
 * Note that these are requests for refund, as refunding a shipment requires 
 * proper verification before being actioned (i.e. ensure label has not been used, for example).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for RequestShipmentRefund Request.  <br>
 *  USAGE:<pre>
 *        RequestNCShipmentRefundClient client = new RequestNCShipmentRefundClient(username, password);
 *        ClientResponse response = client.createNCShipmentRefundRequest(request);
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class RequestNCShipmentRefundClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/ncshipment/340531309186521749/refund";
    	
		// Create XML Request Object
    	NonContractShipmentRefundRequest shipmentRefundRequest = new NonContractShipmentRefundRequest();
        shipmentRefundRequest.setEmail("user@host.com");
		
		// Execute Request
        RequestNCShipmentRefundClient myClient = new RequestNCShipmentRefundClient(username, password);
        ClientResponse resp = myClient.createNCShipmentRefundRequest(shipmentRefundRequest);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(NonContractShipmentRefundRequestInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches CreateShipmentInfo schema.
            if (entity instanceof NonContractShipmentRefundRequestInfo) {
            	NonContractShipmentRefundRequestInfo ncShipmentRefundRequestInfo = (NonContractShipmentRefundRequestInfo) entity;
            	System.out.println("Service Ticket ID: " + ncShipmentRefundRequestInfo.getServiceTicketId());                
            	System.out.println("Service Ticket Date: " + ncShipmentRefundRequestInfo.getServiceTicketDate());
                
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public RequestNCShipmentRefundClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse createNCShipmentRefundRequest(Object xml) throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource
        		.accept("application/vnd.cpc.ncshipment-v4+xml")
        		.header("Content-Type", "application/vnd.cpc.ncshipment-v4+xml")
        		.header("Source-App", "EXT")
        		.header("Consumer-Id", "1")
        		.acceptLanguage("en-CA")
        		.post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
