package ca.canadapost.cpcdp.pickup;

import ca.canadapost.cpcdp.pickup.generated.messages.Messages;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.ContactInfoType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.Link;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.LocationDetailsType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.OnDemandPickupTimeType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupLocationType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestDetails;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestHeaderType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestInfo;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequests;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupTimesType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupTypeType;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;
import com.sun.jersey.api.client.filter.LoggingFilter;

import javax.xml.bind.JAXBContext;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the CreatePickupRequest Canada Post service.
 * <p>
 * The CreatePickupRequest service creates a request for a one-time on-demand pickup or to set up scheduled pickups.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class CreatePickupRequestClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) throws DatatypeConfigurationException {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Pickup_Java_Samples/user.properties
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");

    	LINK = "https://ct.soa-gw.canadapost.ca/enab/" + mailedBy + "/pickuprequest";
    	
    	//create request
    	PickupLocationType location = new PickupLocationType();
    	location.setBusinessAddressFlag(true);
    	
    	ContactInfoType contactInfo = new ContactInfoType();
    	contactInfo.setContactName("John Smith");
    	contactInfo.setEmail("john.smith@host.com");
    	contactInfo.setContactPhone("123-456-7890");
    	
    	LocationDetailsType locationDetails = new LocationDetailsType();
    	locationDetails.setPickupInstructions("Door at back.");
    	
    	OnDemandPickupTimeType onDemand = new OnDemandPickupTimeType();
    	DatatypeFactory df = DatatypeFactory.newInstance();
    	onDemand.setDate(df.newXMLGregorianCalendar("2015-05-14"));
    	onDemand.setPreferredTime("16:00");
    	onDemand.setClosingTime("17:00");
    	
    	PickupTimesType pickupTimes = new PickupTimesType();
    	pickupTimes.setOnDemandPickupTime(onDemand);
    	
    	PickupRequestDetails pickupRequest = new PickupRequestDetails();
    	pickupRequest.setPickupType(PickupTypeType.ON_DEMAND);
    	pickupRequest.setPickupLocation(location);
    	pickupRequest.setContactInfo(contactInfo);
    	pickupRequest.setLocationDetails(locationDetails);
    	pickupRequest.setPickupVolume("50 parcels, 10 packets");
    	pickupRequest.setPickupTimes(pickupTimes);
    	
		// Execute GetPickupAvailability Request
    	CreatePickupRequestClient myClient = new CreatePickupRequestClient(username, password);
        ClientResponse resp = myClient.createPickupRequest(pickupRequest);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(PickupRequestInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches PickupRequestInfo schema.
            if (entity instanceof PickupRequestInfo) {
            	PickupRequestInfo pickupRequestInfo = (PickupRequestInfo) entity;
            	PickupRequestHeaderType header = pickupRequestInfo.getPickupRequestHeader();
            	System.out.println("Request Id: " + header.getRequestId());
            	System.out.println("Request Status: " + header.getRequestStatus());
            	System.out.println("Pickup Type: " + header.getPickupType());
            	System.out.println("Request Date: " + header.getRequestDate());
            	
                for (Iterator<Link> iter = pickupRequestInfo.getLinks().getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public CreatePickupRequestClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
        //aClient.addFilter(new LoggingFilter(System.out));
    }

    public ClientResponse createPickupRequest(Object xml) throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource
        		.accept("application/vnd.cpc.pickuprequest+xml")
        		.header("Content-Type", "application/vnd.cpc.pickuprequest+xml")
        		.acceptLanguage("en-CA")
        		.post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
