package ca.canadapost.cpcdp.pickup;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import javax.xml.bind.JAXBContext;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.client.config.DefaultClientConfig;

import ca.canadapost.cpcdp.pickup.generated.messages.Messages;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupDetails;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestPrice;

/**
 * Sample code for the GetPickupPriceRequest Canada Post service.
 * <p>
 * The GetPickupPriceRequest service returns the price of an on-demand pickup.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class GetPickupPriceRequestClient {
	private Client aClient;
    private static String LINK;
    
    public static void main(String[] args) throws DatatypeConfigurationException {
    	// Your username and password are imported from the following file
    	// CPCWS_Pickup_Java_Samples/user.properties
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");

    	LINK = "https://ct.soa-gw.canadapost.ca/enab/" + mailedBy + "/pickuprequest/price";
    	
    	//create request
    	PickupDetails request = new PickupDetails();
    	DatatypeFactory df = DatatypeFactory.newInstance();
    	request.setDate(df.newXMLGregorianCalendar("2019-02-18"));
    	
    	// Execute GetPickupPriceRequest Request
    	GetPickupPriceRequestClient myClient = new GetPickupPriceRequestClient(username, password);
        ClientResponse resp = myClient.createPickupPriceRequest(request);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());
    	
        //Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(PickupRequestPrice.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches PickupRequestPrice schema.
            if (entity instanceof PickupRequestPrice) {
            	PickupRequestPrice pickupRequestPrice = (PickupRequestPrice) entity;
            	System.out.println("Pre-tax Amount: " + pickupRequestPrice.getPreTaxAmount());
            	System.out.println("GST Amount: " + pickupRequestPrice.getGstAmount());
            	System.out.println("PST Amount: " + pickupRequestPrice.getPstAmount());
            	System.out.println("HST Amount: " + pickupRequestPrice.getHstAmount());
            	System.out.println("Due Amount: " + pickupRequestPrice.getDueAmount());
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    	
    }
    
    public GetPickupPriceRequestClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
        //aClient.addFilter(new LoggingFilter(System.out));
    }

    public ClientResponse createPickupPriceRequest(Object xml) throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource
        		.accept("application/vnd.cpc.pickuprequest+xml")
        		.header("Content-Type", "application/vnd.cpc.pickuprequest+xml")
        		.acceptLanguage("en-CA")
        		.post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
