package ca.canadapost.cpcdp.pickup;

import ca.canadapost.cpcdp.pickup.generated.messages.Messages;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.Link;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestHeaderType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequests;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;
import com.sun.jersey.api.client.filter.LoggingFilter;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.helpers.DefaultValidationEventHandler;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetPickupRequests Canada Post service.
 * <p>
 * The GetPickupRequests service retrieves links to existing open pickup requests. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class GetPickupRequestsClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Pickup_Java_Samples/user.properties
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");

    	LINK = "https://ct.soa-gw.canadapost.ca/enab/" + mailedBy + "/pickuprequest";
    	
		// Execute GetPickupAvailability Request
    	GetPickupRequestsClient myClient = new GetPickupRequestsClient(username, password);
        ClientResponse resp = myClient.getPickupRequests();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus() + "\n");

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(PickupRequests.class, Messages.class);
        	
        	Unmarshaller um = jc.createUnmarshaller();
        	Object entity = jc.createUnmarshaller().unmarshal(respIS);
            
            // Determine whether response data matches PickupRequests schema.
            if (entity instanceof PickupRequests) {
            	PickupRequests pickupRequests = (PickupRequests) entity;
            	for(PickupRequestType pickup : pickupRequests.getPickupRequests()){
                	PickupRequestHeaderType header = pickup.getPickupRequestHeader();
                	System.out.println("Request Id: " + header.getRequestId());
                	System.out.println("Request Status: " + header.getRequestStatus());
                	System.out.println("Pickup Type: " + header.getPickupType());
                	System.out.println("Request Date: " + header.getRequestDate());
                	for(Link link : pickup.getLinks().getLinks()){
                		System.out.println("rel: " + link.getRel() + "\thref: " + link.getHref());
                	}
                	System.out.println("");
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetPickupRequestsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
        //aClient.addFilter(new LoggingFilter(System.out));
    }

    public ClientResponse getPickupRequests() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.pickuprequest+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
