package ca.canadapost.cpcdp.platforms;

import ca.canadapost.cpcdp.platforms.generated.messages.Messages;
import ca.canadapost.cpcdp.platforms.generated.merchantregistration.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetMerchantRegistrationInfo Canada Post service.
 * <p>
 * The GetMerchantRegistrationInfo service is called by the ecommerce platform after 
 * the merchant has completed the Canada Post sign-up process. This call returns
 * merchant data such as customer number and merchant username and password. This
 * information is necessary for the platform to perform web service shipping
 * transactions for the merchant.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetMerchantRegistrationInfo Request.  <br>
 *  USAGE:<pre>
 *        GetMerchantRegistrationInfoClient client = new GetMerchantRegistrationInfoClient(username, password);
 *        ClientResponse response = client.getMerchantRegistrationInfo();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */

public class GetMerchantRegistrationInfoClient {

    private Client aClient;
    
    // REST URI
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/ot/token/1111111111111111111111";
    
    // Main class for local testing only
    public static void main(String[] args) {

    	// Your username and password are imported from the following file
    	// CPCWS_Platforms_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
						
		// Execute GetMerchantDetail Request
        GetMerchantRegistrationInfoClient myClient = new GetMerchantRegistrationInfoClient(username, password);
        ClientResponse resp = myClient.getMerchantRegistrationInfo();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
			jc = JAXBContext.newInstance( MerchantInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Merchant schema.
            if (entity instanceof MerchantInfo) {            	
            	MerchantInfo merchantInfo = (MerchantInfo) entity;   
            	if (merchantInfo.getCustomerNumber() != null) {            		
                	System.out.println("Customer Number: " + merchantInfo.getCustomerNumber());            		
            	}
            	if (merchantInfo.getContractNumber() != null) {            		
                	System.out.println("Contract Number: " + merchantInfo.getContractNumber());            		
            	}
            	if (merchantInfo.getMerchantUsername() != null) {            		
                	System.out.println("Merchant Username: " + merchantInfo.getMerchantUsername());            		
            	}
            	if (merchantInfo.getMerchantPassword() != null) {            		
                	System.out.println("Merchant Password: " + merchantInfo.getMerchantPassword());            		
            	}
            	System.out.println("Has Default CC: " + merchantInfo.isHasDefaultCreditCard());            		

            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetMerchantRegistrationInfoClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getMerchantRegistrationInfo() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.registration+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }	
	
}
