package ca.canadapost.cpcdp.platforms;

import ca.canadapost.cpcdp.platforms.generated.messages.Messages;
import ca.canadapost.cpcdp.platforms.generated.merchantregistration.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetMerchantRegistrationToken Canada Post service.
 * <p>
 * The GetMerchantRegistrationToken service returns a unique registration token that is used
 * to launch a merchant into the Canada Post sign-up process.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetMerchantRegistrationToken Request.  <br>
 *  USAGE:<pre>
 *        GetMerchantRegistrationTokenClient client = new GetMerchantRegistrationTokenClient(username, password);
 *        ClientResponse response = client.getMerchantRegistrationToken();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */

public class GetMerchantRegistrationTokenClient {

    private Client aClient;
    
    // REST URI
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/ot/token";
    
    // Main class for local testing only
    public static void main(String[] args) {

    	// Your username and password are imported from the following file
    	// CPCWS_Platforms_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
						
		// Execute GetMerchantDetail Request
        GetMerchantRegistrationTokenClient myClient = new GetMerchantRegistrationTokenClient(username, password);
        ClientResponse resp = myClient.getMerchantRegistrationToken();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
			jc = JAXBContext.newInstance( Token.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Merchant schema.
            if (entity instanceof Token) {            	
            	Token token = (Token) entity;   
            	System.out.println("Token Id: " + token.getTokenId());            		

            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetMerchantRegistrationTokenClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getMerchantRegistrationToken() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.registration+xml").acceptLanguage("en-CA").post(ClientResponse.class, "");
    }

    public void close() {
        aClient.destroy();
    }	
	
}
