package ca.canadapost.cpcdp.returns;

import ca.canadapost.cpcdp.returns.generated.messages.Messages;
import ca.canadapost.cpcdp.returns.generated.openreturn.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetOpenReturnTemplates Canada Post service.
 * <p>
 * The GetOpenReturnTemplates service is used to retrieve the list of active open
 * return templates (based on the mailed-on-behalf-of customer number). Active
 * open return templates are those that still have open return artifacts available
 * for retrieval. Once all of the artifacts from a template have been retrieved,
 * the open return template is consider depleted, and is deleted.  
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 * Jersey REST client using JAXB for GetOpenReturnTemplates Request.  <br>
 *  USAGE:<pre>
 *        GetOpenReturnTemplatesClient client = new GetOpenReturnTemplatesClient(username, password);
 *        ClientResponse response = client.getOpenReturnTemplates();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetOpenReturnTemplatesClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Returns_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// REST URI
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/openreturn?from=20111201&to=20120301";		
		
		// Execute GetOpenReturnTemplates Request
        GetOpenReturnTemplatesClient myClient = new GetOpenReturnTemplatesClient(username, password);
        ClientResponse resp = myClient.getOpenReturnTemplates();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(Templates.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Templates schema.
            if (entity instanceof Templates) {
            	Templates openreturns = (Templates) entity;
            	if ( openreturns.getLinks().isEmpty() != true ) {
	                for (Iterator<Link> iter = openreturns.getLinks().iterator(); iter.hasNext();) {
	                	Link link = (Link) iter.next();
	                    System.out.println(link.getRel() + ": " + link.getHref());
	                }            	
            	} else {
            		System.out.println("No templates returned.");            		
            	}
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetOpenReturnTemplatesClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getOpenReturnTemplates() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.openreturn-v2+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
