package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.shipment.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetGroups Canada Post service.
 * <p> 
 * The GetGroups service is used to retrieve the list of unique group-id 
 * codes associated with a customer (or mailed-on-behalf-of customer) for which 
 * there are shipments created that have not yet been transmitted.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetGroups Request.  <br>
 *  USAGE:<pre>
 *        GetGroupsClient client = new GetGroupsClient(username, password);
 *        ClientResponse response = client.getGroups();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetGroupsClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// REST URI
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/group";		
		
		// Execute GetGroups Request
        GetGroupsClient myClient = new GetGroupsClient(username, password);
        ClientResponse resp = myClient.getGroups();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(Groups.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Groups schema.
            if (entity instanceof Groups) {
            	Groups groups = (Groups) entity;
            	if (groups.getGroups().isEmpty() == false) {
	                for (Iterator<GroupType> iter = groups.getGroups().iterator(); iter.hasNext();) {
	                	GroupType group = (GroupType) iter.next();
	                    System.out.println(group.getLink().getRel() +  ": " + group.getLink().getHref());
	                }     
            	} else {
            		System.out.println("No groups returned.");
            	}
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetGroupsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getGroups() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.shipment-v8+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
