package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.manifest.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetManifests Canada Post service.
 * <p> 
 * The GetManifests service is used to retrieve a list of manifests (within 
 * a given date range) that were previously transmitted.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetManifests Request.  <br>
 *  USAGE:<pre>
 *        GetManifestsClient client = new GetManifestsClient(username, password);
 *        ClientResponse response = client.getManifests();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetManifestsClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// REST URI
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/manifest?start=20010101";		
		
		// Execute GetManifests Request
        GetManifestsClient myClient = new GetManifestsClient(username, password);
        ClientResponse resp = myClient.getManifests();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(Manifests.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Manifests schema.
            if (entity instanceof Manifests) {
            	Manifests manifests = (Manifests) entity;
            	if ( manifests.getLinks().isEmpty() != true ) {
	                for (Iterator<Link> iter = manifests.getLinks().iterator(); iter.hasNext();) {
	                	Link link = (Link) iter.next();
	                    System.out.println(link.getRel() + ": " + link.getHref());
	                }            	
            	} else {
            		System.out.println("No manifests returned.");
            	}
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetManifestsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getManifests() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.manifest-v8+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
