package ca.canadapost.cpcdp.tracking;

import ca.canadapost.cpcdp.tracking.generated.messages.Messages;
import ca.canadapost.cpcdp.tracking.generated.track.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetTrackingSummary Canada Post service.
 * <p>
 * The GetTrackingSummary service returns the most recent/significant event for a 
 * parcel. If it has been delivered, the delivery details are returned. (The parcel 
 * is identified using PIN, DNC or the other reference parameters).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetTrackingSummary Request.  <br>
 *  USAGE:<pre>
 *        GetTrackingSummaryClient client = new GetTrackingSummaryClient(username, password);
 *        ClientResponse response = client.getTrackingSummary();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetTrackingSummaryClient {

    private Client aClient;
    
    // PIN Summary URI
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/vis/track/pin/1681334332936901/summary";
    // DNC Summary URI
    // private static final String LINK = "https://ct.soa-gw.canadapost.ca/vis/track/dnc/315052413796541/summary";
    // REF Summary URI
    // private static final String LINK = "https://ct.soa-gw.canadapost.ca/vis/track/ref/summary?mailingDateTo=2011-06-25&destinationPostalCode=K2H7X3&mailingDateFrom=2011-08-23&referenceNumber=DIA101";
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Tracking_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
						
		// Execute GetTrackingSummary Request
        GetTrackingSummaryClient myClient = new GetTrackingSummaryClient(username, password);
        ClientResponse resp = myClient.getTrackingSummary();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(TrackingSummary.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches TrackingSummary schema.
            if (entity instanceof TrackingSummary) {            	
            	TrackingSummary trackingSummary = (TrackingSummary) entity;           	
                for (Iterator<PinSummary> iter = trackingSummary.getPinSummaries().iterator(); iter.hasNext();) {
                	PinSummary pinSummary = (PinSummary) iter.next();

                	System.out.println("PIN Number: " + pinSummary.getPin());
	                System.out.println("Mailed On Date: " + pinSummary.getMailedOnDate());
	                System.out.println("Event Description: " + pinSummary.getEventDescription());
	                System.out.println();
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetTrackingSummaryClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getTrackingSummary() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.track+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
